--
-- IBM Confidential
-- OCO Source Materials
-- 5724-U98, 5655-V48, 5724-U50, 5655-V46
-- (C) Copyright IBM Corp. 2011
-- The source code for this program is not published or other -
-- wise divested of its trade secrets, irrespective of what has
-- been deposited with the U.S. Copyright Office.
--

--
-- This file contains the Optim Design Directory DDL.
--

--
-- The directory_properties table contains Design Directory properties needed by the infrastructure.
-- Notes:
-- Neither the name nor the value column is intended to be internationalizable.
--
--
-- IBM Confidential
-- OCO Source Materials
-- 5724-U98, 5655-V48, 5724-U50, 5655-V46
-- (C) Copyright IBM Corp. 2011
-- The source code for this program is not published or other -
-- wise divested of its trade secrets, irrespective of what has
-- been deposited with the U.S. Copyright Office.
--

--
-- This file contains the Optim Design Directory DDL.
--

--
-- The directory_properties table contains Design Directory properties needed by the infrastructure.
-- Notes:
-- Neither the name nor the value column is intended to be internationalizable.
--
create table dd_directory_properties (
    name            varchar2(128) not null,
    value           varchar2(128) not null,
	constraint directory_prop_pk primary key (name)
);

--
-- The folders table contains a hierarchy of folders allowing the user to organize their work.
-- Notes:
-- The id column is always a 36-character, US-ASCII UUID.
-- The parent_id column is always a 36-character, US-ASCII UUID.
-- The name and description columns are assumed to be in a local language, so may hold wide characters.
--
create table dd_folders (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    folder_type     integer not null,
    parent_id       char(36),
    constraint folders_pk primary key (id),
    constraint folders_name unique (name, parent_id),
    constraint folders_fk foreign key (parent_id) references dd_folders (id)
);

--
-- The global policies table store all policy binding which can be shared between multiple entries
--
create table dd_global_policies (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    policy_id       nvarchar2(128) not null,
    policy_type     nvarchar2(128) not null,
    policy_kind     nvarchar2(128) not null,
    is_local        char(1) default 'f',
    folder_id       	char(36),
    constraint global_policies_pk primary key (id),
    constraint global_policies_fk1 foreign key (folder_id) references dd_folders (id)
);

--
-- The data_stores table contains data store information.
-- Notes:
-- The id is always a 36-character, US-ASCII UUID.
-- The name and description columns are assumed to be in a local language, so may hold wide characters.
--
create table dd_data_stores (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    type			varchar2(32) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    constraint data_stores_pk primary key (id)    
);

create table dd_entities (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    schema_name     nvarchar2(128) not null,
    data_store_name nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    constraint entities_pk primary key (id) 
);

create table dd_entity_policies (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    entity_id       char(36) not null,
    policy_id       nvarchar2(128) not null,
    policy_type     nvarchar2(128) not null,
    policy_kind     nvarchar2(128) not null,
    is_local        char(1) default 'f',
    constraint entity_policies_pk primary key (id),
    constraint entity_policies_fk1 foreign key (entity_id) references dd_entities (id) on delete cascade
);

create table dd_policy_metadata (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    policy_id       nvarchar2(128) not null,
    policy_type     nvarchar2(128) not null,
    policy_kind     nvarchar2(128) not null,
    constraint policy_metadata_pk primary key (id)
);

create table dd_relationships (
    id                  char(36) not null,
    version             int not null,
    parent_id       	char(36) not null,
    child_id        	char(36) not null,
    create_time     	timestamp not null,
    create_user     	varchar2(32) not null,
    update_time     	timestamp,
    update_user     	varchar2(32),
    name            	nvarchar2(128) not null,
    description         nvarchar2(255),
    object_state	    varchar2(36) not null,
    relationship_type   integer not null,
    constraint relationships_pk primary key (name, parent_id, child_id),
    constraint relationships_fk1 foreign key (parent_id) references dd_entities (id) on delete cascade,
    constraint relationships_fk2 foreign key (child_id) references dd_entities (id) on delete cascade
);

create table dd_point_and_shoot_lists (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    entity_id       char(36) not null,
    is_local        char (1),
    constraint point_and_shoot_lists_pk primary key (id),
    constraint point_and_shoot_list_fk1 foreign key (entity_id) references dd_entities (id) on delete cascade
);

create table dd_data_access_plans (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    folder_id       char(36) not null,
    point_and_shoot_list_id    char(36),
    is_local		char(1) default 'f',
    is_file			char(1) default 'f',
    constraint data_access_plans_pk primary key (id),
    constraint data_access_plans_fk1 foreign key (folder_id) references dd_folders (id) on delete cascade,
    constraint data_access_plans_fk2 foreign key (point_and_shoot_list_id) references dd_point_and_shoot_lists (id)
);

create table dd_dap_policies (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    dap_id          char(36) not null,
    policy_ext_id   nvarchar2(128) not null,
    policy_type     nvarchar2(128) not null,
    constraint dap_policies_pk primary key (id),
    constraint dap_policies_fk1 foreign key (dap_id) references dd_data_access_plans (id) on delete cascade
);

create table dd_dap_policy_entities (
    id              char(36) not null,
    dap_id          char(36) not null,
    policy_id       char(36) not null,
    entity_id       nvarchar2(324) not null,
    policy_ext_id   nvarchar2(128) not null,
    policy_type     nvarchar2(128) not null,
    content_id          char(36),
    constraint dap_policy_entities_pk primary key (id),
    constraint dap_policy_entities_fk1 foreign key (dap_id) references dd_data_access_plans (id) on delete cascade,
    constraint dap_policy_entities_fk2 foreign key (policy_id) references dd_dap_policies (id) on delete cascade
);

--
-- The dd_column_maps table contains column map information.
--
create table dd_column_maps (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    left_entity_id      nvarchar2(324), -- marked nullable to allow easy migration
    right_entity_id     nvarchar2(324), -- marked nullable to allow easy migration
    folder_id       	char(36) not null,
    file_guid          varchar2(38),
    file_time          timestamp,
    is_local			char(1) default 'f',
    constraint column_maps_pk primary key (id),
    constraint column_maps_fk1 foreign key (folder_id) references dd_folders (id) on delete cascade
);

create table dd_column_map_assignments (
	column_map_id				char(36) not null,
	left_attribute  			nvarchar2(128),
	left_attribute_datatype  	nvarchar2(128),
	right_attribute 			nvarchar2(128) not null,
	right_attribute_datatype 	nvarchar2(128) not null,
	column_map_policy_id char(36),
	content_id          char(36),
	constraint column_map_assignments_pk primary key (column_map_id, right_attribute),
    constraint column_map_assignments_fk1 foreign key (column_map_id) references dd_column_maps (id),
    constraint column_map_assignments_fk2 foreign key (column_map_policy_id) references dd_entity_policies (id) on delete cascade
);

--
-- The dd_table_maps table contains table map information.
--
create table dd_table_maps (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    folder_id       char(36) not null,
    dap_id          char(36),
    file_guid       char(38),
    file_time       timestamp,
    is_local		char(1) default 'f',
    constraint table_maps_pk primary key (id),
    constraint table_maps_fk1 foreign key (folder_id) references dd_folders (id) on delete cascade
);

--
-- The dd_table_map_assignments table contains table map assignment information.
--
create table dd_table_map_assignments (
    table_map_id          char(36) not null,
    left_entity_id         nvarchar2(324) not null, 
    right_entity_id		   nvarchar2(324),
    column_map_id 	   char(36),
    archive_action_id 	   char(36),
    content_id          char(36),
    constraint table_map_assignments_pk primary key (table_map_id, left_entity_id),
    constraint table_map_assignments_fk1 foreign key (table_map_id) references dd_table_maps (id),
    constraint table_map_assignments_fk2 foreign key (column_map_id) references dd_column_maps (id) on delete cascade
);

create table dd_services (
    id              	char(36) not null,
    version         	int not null,
    create_time     	timestamp not null,
    create_user     	varchar2(32) not null,
    update_time     	timestamp,
    update_user     	varchar2(32),
    name            	nvarchar2(128) not null,
    description     	nvarchar2(255),
    object_state	    varchar2(36) not null,
    folder_id       	char(36) not null,
    folder_path			clob,    
    dap_id          	char(36),
    table_map_id    	char(36),
    point_and_shoot_list_id    char(36),
    service_type    	nvarchar2(128) not null,
    service_state		varchar2(36) not null,
    service_annotation	varchar2(36),
    product_platform 	nvarchar2(128) not null,
    file_guid       char(38),
    file_time       timestamp,
    is_local		char(1) default 'f',
    constraint services_pk primary key (id),
    constraint services_fk1 foreign key (dap_id) references dd_data_access_plans (id),
    constraint services_fk2 foreign key (folder_id) references dd_folders (id) on delete cascade,
    constraint services_fk3 foreign key (table_map_id) references dd_table_maps (id),
    constraint services_fk4 foreign key (point_and_shoot_list_id) references dd_point_and_shoot_lists (id)
);

create table dd_service_policies (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    service_id      char(36) not null,
    policy_ext_id   nvarchar2(128) not null,
    policy_type     nvarchar2(128) not null,
    constraint service_policies_pk primary key (id),
    constraint service_policies_fk1 foreign key (service_id) references dd_services (id) on delete cascade
);

create table dd_service_dependent_entities (
    service_id          	char(36) not null,
    dependent_service_id    char(36) not null,
    content_id          char(36),
    constraint service_dependent_pk primary key (service_id, dependent_service_id),
    constraint service_dependent_entities_fk1 foreign key (service_id) references dd_services (id) on delete cascade,
    constraint service_dependent_entities_fk2 foreign key (dependent_service_id) references dd_services (id) on delete cascade
);

create table dd_primary_keys (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    is_generic		char(1) default 'f',
    entity_id       char(36),
    constraint primary_keys_pk primary key (id),
    constraint primary_key_fk1 foreign key (entity_id) references dd_entities (id) on delete cascade
    
);

create table dd_jdbc_drivers (
    id              char(36) not null,
    version         int not null,
    create_time     timestamp not null,
    create_user     varchar2(32) not null,
    update_time     timestamp,
    update_user     varchar2(32),
    name            nvarchar2(128) not null,
    description     nvarchar2(255),
    object_state	varchar2(36) not null,
    vendor          varchar2(32) not null,
    vendor_version  varchar2(32) not null,
    driver_file	nvarchar2(255) not null,
    constraint jdbc_drivers_pk primary key (id, vendor, vendor_version)
);

create table dd_contents (
    id              varchar2(38) not null,
    content_type	varchar2(36) not null,
    create_time     timestamp not null,
    update_time     timestamp,
    data            blob,
    constraint contents_pk primary key (id)
);